@@ -19,13 +19,8 @@
 
 #include "mod_chxj.h"
 
-extern char* chxj_url_encode(
-  request_rec*           r, 
-  const char*            src);
-
-extern char* chxj_url_decode(
-  request_rec*           r, 
-  const char*            src);
+extern char* chxj_url_encode(apr_pool_t *pool, const char *src);
+extern char* chxj_url_decode(apr_pool_t *pool, const char *src);
 
 #endif
 /*
