@@ -1349,6 +1349,7 @@ s_chtml30_start_form_tag(void *pdoc, Node *node)
   Doc           *doc;
   request_rec   *r;
   Attr          *attr;
+  char          *new_hidden_tag = NULL;
 
   chtml30 = GET_CHTML30(pdoc);
   doc     = chtml30->doc;
@@ -1369,6 +1370,12 @@ s_chtml30_start_form_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       value = chxj_encoding_parameter(r, value);
       value = chxj_add_cookie_parameter(r, value, chtml30->cookie);
+      char *q;
+      q = strchr(value, '?');
+      if (q) {
+        new_hidden_tag = chxj_form_action_to_hidden_tag(doc->pool, value, 0);
+        *q = 0;
+      }
       W_L(" action=\"");
       W_V(value);
       W_L("\"");
@@ -1390,6 +1397,9 @@ s_chtml30_start_form_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  if (new_hidden_tag) {
+    W_V(new_hidden_tag);
+  }
   return chtml30->out;
 }
 
